from argparse import ArgumentParser


def get_args():
    parser = ArgumentParser(description='Transform field(s) to field(s)')
    subparsers = parser.add_subparsers(title='modes', dest='mode', required=True)
    train_parser = subparsers.add_parser('train')
    test_parser = subparsers.add_parser('test')

    add_train_args(train_parser)
    add_test_args(test_parser)

    args = parser.parse_args()

    return args


def add_common_args(parser):
    parser.add_argument('--in-channels', type=int, required=True,
            help='number of input channels')
    parser.add_argument('--out-channels', type=int, required=True,
            help='number of output or target channels')
    parser.add_argument('--norms', type=str_list, help='comma-sep. list '
            'of normalization functions from map2map.data.norms')
    parser.add_argument('--criterion', default='MSELoss',
            help='model criterion from torch.nn')
    parser.add_argument('--load-state', default='', type=str,
            help='path to load model, optimizer, rng, etc.')


def add_train_args(parser):
    add_common_args(parser)

    parser.add_argument('--train-in-patterns', type=str_list, required=True,
            help='comma-sep. list of glob patterns for training input data')
    parser.add_argument('--train-tgt-patterns', type=str_list, required=True,
            help='comma-sep. list of glob patterns for training target data')
    parser.add_argument('--val-in-patterns', type=str_list, required=True,
            help='comma-sep. list of glob patterns for validation input data')
    parser.add_argument('--val-tgt-patterns', type=str_list, required=True,
            help='comma-sep. list of glob patterns for validation target data')
    parser.add_argument('--epochs', default=128, type=int,
            help='total number of epochs to run')
    parser.add_argument('--batches-per-gpu', default=8, type=int,
             help='mini-batch size per GPU')
    parser.add_argument('--loader-workers-per-gpu', default=4, type=int,
            help='number of data loading workers per GPU')
    parser.add_argument('--augment', action='store_true',
            help='enable training data augmentation')
    parser.add_argument('--optimizer', default='Adam',
            help='optimizer from torch.optim')
    parser.add_argument('--lr', default=0.001, type=float,
            help='initial learning rate')
#    parser.add_argument('--momentum', default=0.9, type=float,
#            help='momentum')
#    parser.add_argument('--weight-decay', default=1e-4, type=float,
#            help='weight decay')
    parser.add_argument('--dist-backend', default='nccl', type=str,
            choices=['gloo', 'nccl'], help='distributed backend')
    parser.add_argument('--seed', default=42, type=int,
            help='seed for initializing training')
    parser.add_argument('--log-interval', default=20, type=int,
            help='interval between logging training loss')


def add_test_args(parser):
    add_common_args(parser)

    parser.add_argument('--test-in-patterns', type=str_list, required=True,
            help='comma-sep. list of glob patterns for test input data')
    parser.add_argument('--test-tgt-patterns', type=str_list, required=True,
            help='comma-sep. list of glob patterns for test target data')


def str_list(s):
    return s.split(',')
